@@ -4,7 +4,7 @@
 # Copyright (c) 2003 Jochen Lillich <jl@teamlinux.de>
 ###########################################################
 #
-# $Id: DBI.pm,v 1.6 2003/10/31 08:28:33 jlillich Exp $
+# $Id: DBI.pm 14 2004-09-27 17:43:37Z jlillich $
 #
 
 package CGI::Session::Auth::DBI;
@@ -18,14 +18,6 @@ use DBI;
 
 our $VERSION = do { my @r = (q$Revision: 1.6 $ =~ /\d+/g); sprintf "%d." . "%03d" x (scalar @r - 1), @r; };
 
-# column names in database
-my $COL_USERID = 'userid';
-my $COL_USERNAME = 'username';
-my $COL_PASSWORD = 'passwd';
-my $COL_IPUSERID = 'userid';
-my $COL_IPADDR = 'network';
-my $COL_IPMASK = "netmask";
-
 ###########################################################
 ###
 ### general methods
@@ -51,29 +43,40 @@ sub new {
     #
     # class specific parameters
     #
-    
-    # parameter 'DSN': DBI data source name
-    my $dsn = $params->{DSN} || croak("No DSN parameter");
-    # parameter 'DBUser': database connection username
-    my $dbuser = $params->{DBUser} || '';
-    # parameter 'DBPasswd': database connection password
-    my $dbpasswd = $params->{DBPasswd} || "";
-    # parameter 'DBAttr': optional database connection attributes
-    my $dbattr = $params->{DBAttr} || {};
+
+    if ($params->{DBHandle}) {
+      $self->{dbh} = $params->{DBHandle};
+    } else {
+      # parameter 'DSN': DBI data source name
+      my $dsn = $params->{DSN} || croak("No DSN parameter");
+      # parameter 'DBUser': database connection username
+      my $dbuser = $params->{DBUser} || '';
+      # parameter 'DBPasswd': database connection password
+      my $dbpasswd = $params->{DBPasswd} || "";
+      # parameter 'DBAttr': optional database connection attributes
+      my $dbattr = $params->{DBAttr} || {};
+      # database handle
+      $self->{dbh} = DBI->connect($dsn, $dbuser, $dbpasswd, $dbattr) or croak("DB error: " . $DBI::errstr);
+    }
     # parameter 'UserTable': name of user data table
     $self->{usertable} = $params->{UserTable} || 'auth_user';
+    $self->{usernamefield} = $params->{UsernameField} || 'username';
+    $self->{passwordfield} = $params->{PasswordField} || 'passwd';
+    $self->{useridfield} = $params->{UserIDField} || 'userid';
     # parameter 'GroupTable': name of user data table
     $self->{grouptable} = $params->{GroupTable} || 'auth_group';
+    $self->{groupfield} = $params->{GroupField} || 'group';
+    $self->{groupuseridfield} = $params->{GroupUserIDField} || 'userid';
     # parameter 'IPTable': name of ip network table
     $self->{iptable} = $params->{IPTable} || 'auth_ip';
+    $self->{ipuseridfield} = $params->{IPUserIDField} || 'userid';
+    $self->{ipaddressfield} = $params->{IPAddressField} || 'network';
+    $self->{ipmaskfield} = $params->{IPNetMaskField} || 'netmask';
     
     #
     # class members
     #
     
-    # database handle
-    $self->{dbh} = DBI->connect($dsn, $dbuser, $dbpasswd, $dbattr) or croak("DB error: " . $DBI::errstr);
-    
     # blessed are the greek
     bless($self, $class);
     
@@ -104,8 +107,8 @@ sub _login {
     my $query = sprintf(
         "SELECT * FROM %s WHERE %s = ? AND %s = ?",
         $self->{usertable},
-        $COL_USERNAME,
-        $COL_PASSWORD
+        $self->{usernamefield},
+        $self->{passwordfield},
     );
     $self->_debug("query: $query");
     # search for username
@@ -141,9 +144,9 @@ sub _ipAuth {
     
     my $query = sprintf(
         "SELECT %s, %s, %s FROM %s",
-        $COL_IPUSERID,
-        $COL_IPADDR,
-        $COL_IPMASK,
+        $self->{ipuseridfield},
+        $self->{ipaddressfield},
+        $self->{ipmaskfield},
         $self->{iptable}
     );
     $self->_debug("query: $query");
@@ -153,12 +156,12 @@ sub _ipAuth {
     $sth->execute or croak $self->_dbh()->errstr;
     while (my $rec = $sth->fetchrow_hashref) {
         
-        $self->_debug("compare IP network ", $rec->{$COL_IPADDR}, "/", $rec->{$COL_IPMASK});
+        $self->_debug("compare IP network ", $rec->{$self->{ipaddressfield}}, "/", $rec->{$self->{ipmaskfield}});
         
-        if ($remoteip->within(new NetAddr::IP( $rec->{$COL_IPADDR}, $rec->{$COL_IPMASK}))) {
+        if ($remoteip->within(new NetAddr::IP( $rec->{$self->{ipaddressfield}}, $rec->{$self->{ipmaskfield}}))) {
             $self->_debug("we have a winner!");
             # get user record
-            my $user = $self->_getUserRecord($rec->{$COL_IPUSERID});
+            my $user = $self->_getUserRecord($rec->{$self->{ipuseridfield}});
             $self->_extractProfile($user);
             $result = 1;
             last;
@@ -185,8 +188,9 @@ sub _loadProfile {
     my ($userid) = @_;
     
     my $query = sprintf(
-        "SELECT * FROM %s WHERE userid = ?",
-        $self->{usertable}
+        "SELECT * FROM %s WHERE %s = ?",
+        $self->{usertable},
+        $self->{useridfield}
     );
     $self->_debug("query: $query");
     my $sth = $self->_dbh->prepare($query);
@@ -212,13 +216,13 @@ sub saveProfile {
     my @values;
     my $first = 1;
 	foreach (keys %{$self->{profile}}) {
-		if ($_ ne $COL_USERID) {
+		if ($_ ne $self->{useridfield}) {
 			$query .= (($first) ? '' : ', ') . $_ . " = ?";
 			push @values, $self->{profile}{$_};
 			$first = 0;
 		}
 	}
-	$query .= " WHERE " . $COL_USERID . " = ?";
+	$query .= " WHERE " . $self->{useridfield} . " = ?";
 	$self->_debug("update query: ", $query);
 
     my $sth = $self->_dbh()->prepare($query);
@@ -234,6 +238,31 @@ sub isGroupMember {
     ## check if user is in given group
     ##
     
+    my $self = shift;
+    my ($group) = @_;
+    
+    $self->_debug("group: $group");
+    
+    my $result = 0;
+    
+    my $query = sprintf(
+        "SELECT * FROM %s WHERE %s = ? AND %s = ?",
+        $self->{grouptable},
+        $self->{groupuseridfield},
+        $self->{groupfield}
+    );
+    $self->_debug("query: $query");
+    $self->_debug("values: $self->{userid}, $group");
+    # search for username
+    my $sth = $self->_dbh->prepare($query);
+    $sth->execute($self->{userid}, $group) or croak $self->_dbh->errstr;
+    if (my $rec = $sth->fetchrow_hashref) {
+        $self->_debug("found group entry");
+        $result = 1;
+    }
+    $sth->finish;
+    
+    return $result;
 }
 
 ###########################################################
@@ -266,7 +295,7 @@ sub _extractProfile {
     my $self = shift;
     my ($rec) = @_;
     
-    $self->{userid} = $rec->{$COL_USERID};
+    $self->{userid} = $rec->{$self->{useridfield}};
     foreach ( keys %$rec ) {
         $self->{profile}{$_} = $rec->{$_};
     }
@@ -288,12 +317,12 @@ sub _getUserRecord {
     my $query = sprintf(
         "SELECT * FROM %s WHERE %s = ?",
         $self->{usertable},
-        $COL_USERID
+        $self->{useridfield}
     );
     $self->_debug("query: $query");
     # search for username
     my $sth = $self->_dbh->prepare($query);
-    $sth->execute($userid) or croak _dbh->errstr;
+    $sth->execute($userid) or croak $self->_dbh->errstr;
     
     return $sth->fetchrow_hashref;
 }
@@ -337,7 +366,7 @@ CGI::Session::Auth::DBI - Authenticated sessions for CGI scripts
 
 =head1 DESCRIPTION
 
-CGI::Session::Auth::DBI is a subclass of L<DBI::Session::Auth>. It uses a
+CGI::Session::Auth::DBI is a subclass of L<CGI::Session::Auth>. It uses a
 relational database for storing the authentication data, using the L<DBI> module
 as database interface.
 
@@ -371,7 +400,7 @@ All additional columns will also be stored and accessible as user profile fields
 
 C<userid> is a 32-character string and can be generated randomly by
 
-perl -MCGI::Session::Auth -e 'print CGI::Session::Auth::_uniqueUserID("myname"), "\n";'
+perl -MCGI::Session::Auth -e 'print CGI::Session::Auth::uniqueUserID("myname"), "\n";'
 
 The C<auth_ip> table is used for IP address based authentication. Every row combines a pair of network
 address and subnet mask (both in dotted quad notation) with a user ID. The C<userid> column
@@ -384,7 +413,9 @@ all CGI::Session::Auth classes, CGI::Session::Auth::DBI understands the followin
 
 =over 1
 
-=item B<DSN>: Data source name for the database connection (mandatory).
+=item B<DBHandle>: Active database handle.  For an explanation, see the L<DBI> documentation.
+
+=item B<DSN>: Data source name for the database connection.
 For an explanation, see the L<DBI> documentation.
 
 =item B<DBUser>: Name of the user account used for the database connection. (Default: none)
